Package: rco
Type: Package
Title: The R Code Optimizer
Version: 1.0.2
Authors@R: c(
    person(given = "Juan Cruz", family = "Rodriguez", 
        role = c("aut", "cre"), email = "jcrodriguez@unc.edu.ar"),
    person("Yihui", "Xie", role = "ctb", email = "xie@yihui.name", comment = 
        c(ORCID = "0000-0003-0645-5666")),
    person("Nicolás", "Wolovick", role = "ctb", email = 
        "nicolasw@famaf.unc.edu.ar")
    )
Maintainer: Juan Cruz Rodriguez <jcrodriguez@unc.edu.ar>
Description: Automatically apply different strategies to optimize R code. 
    'rco' functions take R code as input, and returns R code as output.
Depends: R (>= 3.6.0)
License: GPL-3
URL: https://jcrodriguez1989.github.io/rco/
BugReports: https://github.com/jcrodriguez1989/rco/issues
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.1.1
Suggests: covr, diffr, ggplot2, httr, knitr, markdown, microbenchmark,
        rmarkdown, rstudioapi, rvest, shiny, shinythemes, testthat,
        xml2
NeedsCompilation: no
Packaged: 2021-04-17 13:37:11 UTC; jcrodriguez
Author: Juan Cruz Rodriguez [aut, cre],
  Yihui Xie [ctb] (<https://orcid.org/0000-0003-0645-5666>),
  Nicolás Wolovick [ctb]
Repository: CRAN
Date/Publication: 2021-04-17 14:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 21:36:50 UTC; unix
